<?php
/*
Plugin Name: Comment E-Mail Verification
Plugin URI: http://ten-fingers-and-a-brain.com/wordpress-plugins/comment-email-verify/
Version: 0.1.2.1
Description: If a comment is held for moderation an e-mail message is sent to the comment's author with a link to verify the comment author's e-mail address. When the comment author clicks on that link the comment gets approved immediately. This makes discussions more lively as users don't have to wait for the blog admin to approve the comment.
Author: Martin Lormes
Author URI: http://ten-fingers-and-a-brain.com/
Text Domain: comment-email-verify
*/
/*
Copyright (c) 2010 Martin Lormes

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program. If not, see <http://www.gnu.org/licenses/>.
*/
/** Comment E-Mail Verification (WordPress Plugin) */

// i18n/l10n
load_plugin_textdomain ( 'comment-email-verify', '', basename ( dirname ( __FILE__ ) ) );

/** Comment E-Mail Verification (WordPress Plugin) functions wrapped in a class. (namespacing pre PHP 5.3) */
class comment_email_verify
{

  /** hooked to {@link http://codex.wordpress.org/Plugin_API/Action_Reference WordPress action}: {@link http://codex.wordpress.org/Plugin_API/Action_Reference/comment_post comment_post} */
  function comment_post ( $id, $status )
  {
    if ( '0' == $status ) // todo: don't do this for trackbacks and pingbacks
    {
      // get plugin options
      $options = get_option ( 'comment-email-verify' );
      
      // create verification key and url
      $chars = '1234567890qwertzuiopasdfghjklyxcvbnm';
      $key = '';
      for($i=0;$i<6;$i++) $key .= substr ( $chars, rand ( 0, strlen ( $chars ) - 1 ), 1 );
      update_comment_meta($id,'comment_email_verify',$key);
      $verification_url = get_bloginfo('url')."/?comment_email_verify=$id+$key";
      
      // get data we need for the shortcodes
      $blogname = wp_specialchars_decode(get_option('blogname'), ENT_QUOTES);
      $blogdescription = wp_specialchars_decode(get_option('blogdescription'), ENT_QUOTES);
      $home = get_bloginfo('url');
      
      $thecomment = get_comment($id);
      
      $author_name = $thecomment->comment_author;
      $author_email = $thecomment->comment_author_email;
      $author_url = $thecomment->comment_author_url;
      $author_ip = $thecomment->comment_author_IP;
      $author_hostname = gethostbyaddr ( $author_ip );
      $comment = $thecomment->comment_content;
      
      $thepost = get_post($thecomment->comment_post_ID);
      
      $post_title = $thepost->post_title;
      $post_permalink = get_permalink($thecomment->comment_post_ID);
      
      if ( !$options or !isset ( $options['template'] ) or !isset ( $options['template']['subject'] ) or '' == trim ( $options['template']['subject'] ) )
      {
        $subject = __( '[[blogname]] Please verify your e-mail address', 'comment-email-verify' );
      }
      else
      {
        $subject = $options['template']['subject'];
      }
      
      $subject = preg_replace (
        array (
          '/\[blogname]/i',
          '/\[post-title]/i',
        ),
        array (
          $blogname,
          $post_title,
        ),
        $subject
      );
      
      if ( !$options or !isset ( $options['template'] ) or !isset ( $options['template']['body'] ) or '' == trim ( $options['template']['body'] ) )
      {
        $body = __( "Dear [author-name],\n\nthanks for replying to [post-title] ([post-permalink]) on [blogname] - [blogdescription] ([home]).\n\nPlease verify your email address by clicking on this link:\n[verification-url]\n\nThis is your comment:\n\n[comment]\n\n--\nThis comment was posted from [author-ip] - [author-hostname]", 'comment-email-verify' );
      }
      else
      {
        $body = $options['template']['body'];
      }
      
      if ( !preg_match ( '/\[verification-url]/i', $body ) ) $body .= "\n\n$verification_url";
      $body = preg_replace (
        array (
          '/\[verification-url]/i',
          '/\[blogname]/i',
          '/\[blogdescription]/i',
          '/\[home]/i',
          '/\[post-title]/i',
          '/\[post-permalink]/i',
          '/\[author-name]/i',
          '/\[author-email]/i',
          '/\[author-url]/i',
          '/\[author-ip]/i',
          '/\[author-hostname]/i',
          '/\[comment]/i',
        ),
        array (
          $verification_url,
          $blogname,
          $blogdescription,
          $home,
          $post_title,
          $post_permalink,
          $author_name,
          $author_email,
          $author_url,
          $author_ip,
          $author_hostname,
          $comment,
        ),
        $body
      );
      
      wp_mail($author_email,$subject,$body);
    }
  }

  /** hooked to {@link http://codex.wordpress.org/Plugin_API/Action_Reference WordPress action}: {@link http://codex.wordpress.org/Plugin_API/Action_Reference/init init} */
  function init ()
  {
    add_action ( 'comment_post', array ( 'comment_email_verify', 'comment_post' ), 10, 2 );
    
    if ( 'GET' != $_SERVER['REQUEST_METHOD'] or empty ( $_GET['comment_email_verify'] ) ) return;
    
    $code = $_GET['comment_email_verify'];
    list ( $id, $key ) = explode ( ' ', $code, 2 );
    
    if ( get_comment_meta($id,'comment_email_verify',true) != $key ) wp_die ( __( 'something went wrong. your e-mail address was not verified.', 'comment-email-verify' ) );
    
    $comment = get_comment($id);
    $status = $comment->comment_approved;
    if ( ! in_array($status,array(0,1,)) ) wp_die ( __( 'something went wrong. your e-mail address was not verified.', 'comment-email-verify' ) );
    
    global $wpdb;
    $wpdb->update( $wpdb->comments, array('comment_approved' => 1), array('comment_ID' => $id) );
    
    delete_comment_meta($id,'comment_email_verify');
    
    clean_comment_cache($id);
    $comment = get_comment($id);
    wp_update_comment_count($comment->comment_post_ID);
    
    if ( get_option('comments_notify') ) wp_notify_postauthor($id, $comment->comment_type);
    
    wp_redirect(get_comment_link($id));
    exit; // this is necessary because wp_redirect does not seem to work without it
  }

  /** */
  function settings_section__template ()
  {
    _e( '<p>Customize the e-mail message you are sending to comment authors. Leave a field blank to reset it to the default.</p>', 'comment-email-verify' );
  }
  
  /** */
  function settings_section__core_options ()
  {
    echo sprintf ( __( '<p>These are WordPress core settings. They may affect this plugin. They can also be set on <a href="options-discussion.php">%s</a>.</p>', 'comment-email-verify' ), __( 'Discussion Settings' ) );
  }

  /** */
  function settings_field__template__subject ()
  {
    $options = get_option ( 'comment-email-verify' );
    if ( !$options or !isset ( $options['template'] ) or !isset ( $options['template']['subject'] ) or '' == trim ( $options['template']['subject'] ) )
    {
      $subject = __( '[[blogname]] Please verify your e-mail address', 'comment-email-verify' );
    }
    else
    {
      $subject = $options['template']['subject'];
    }
    ?>
    <input type="text" class="regular-text" name="comment-email-verify[template][subject]" value="<?php echo htmlspecialchars ( $subject ); ?>" />
    <span class="description">
      <?php _e( 'You may use the following shortcodes in the subject line: <code>[blogname]</code>, <code>[post-title]</code>', 'comment-email-verify' ); ?>
    </span>
    <?php
  }

  /** */
  function settings_field__template__body ()
  {
    $options = get_option ( 'comment-email-verify' );
    if ( !$options or !isset ( $options['template'] ) or !isset ( $options['template']['body'] ) or '' == trim ( $options['template']['body'] ) )
    {
      $body = __( "Dear [author-name],\n\nthanks for replying to [post-title] ([post-permalink]) on [blogname] - [blogdescription] ([home]).\n\nPlease verify your email address by clicking on this link:\n[verification-url]\n\nThis is your comment:\n\n[comment]\n\n--\nThis comment was posted from [author-ip] - [author-hostname]", 'comment-email-verify' );
    }
    else
    {
      $body = $options['template']['body'];
    }
    ?>
    <textarea rows="10" cols="50" class="large-text" name="comment-email-verify[template][body]"><?php echo htmlspecialchars ( $body ); ?></textarea>
    <span class="description">
      <?php _e( 'You may use the following shortcodes in the message body: <code>[verification-url]</code>*, <code>[blogname]</code>, <code>[blogdescription]</code>, <code>[home]</code>, <code>[post-title]</code>, <code>[post-permalink]</code>, <code>[author-name]</code>, <code>[author-email]</code>, <code>[author-url]</code>, <code>[author-ip]</code>, <code>[author-hostname]</code>, <code>[comment]</code><br/><br/>* If you don\'t use <code>[verification-url]</code> that information will be appended to your message after a blank line.', 'comment-email-verify' ); ?>
    </span>
    <?php
  }
  
  /** */
  function settings_field__core_options__other ()
  {
    $require_name_email = get_option ( 'require_name_email' );
    $comment_registration = get_option ( 'comment_registration' );
    
    ?>
    <input type="checkbox" name="require_name_email" id="require_name_email" value="1"<?php if ( '1' == $require_name_email ) echo ' checked="checked"'; ?> />
    <label for="require_name_email"><?php _e( 'Comment author must fill out name and e-mail' ); // this is in the default domain ?></label>
    <br/>
    <input type="checkbox" name="comment_registration" id="comment_registration" value="1"<?php if ( '1' == $comment_registration ) echo ' checked="checked"'; ?> />
    <label for="comment_registration"><?php _e( 'Users must be registered and logged in to comment' ); // this is in the default domain ?></label>
    <?php
  }

  /** */
  function settings_field__core_options__email ()
  {
    $comments_notify = get_option ( 'comments_notify' );
    $moderation_notify = get_option ( 'moderation_notify' );
    
    ?>
    <input type="checkbox" name="comments_notify" id="comments_notify" value="1"<?php if ( '1' == $comments_notify ) echo ' checked="checked"'; ?> />
    <label for="comments_notify"><?php _e( 'Anyone posts a comment' ); // this is in the default domain ?></label>
    <br/>
    <input type="checkbox" name="moderation_notify" id="moderation_notify" value="1"<?php if ( '1' == $moderation_notify ) echo ' checked="checked"'; ?> />
    <label for="moderation_notify"><?php _e( 'A comment is held for moderation' ); // this is in the default domain ?></label>
    <?php
  }

  /** */
  function settings_field__core_options__before ()
  {
    $comment_moderation = get_option ( 'comment_moderation' );
    $comment_whitelist = get_option ( 'comment_whitelist' );
    
    ?>
    <input type="checkbox" name="comment_moderation" id="comment_moderation" value="1"<?php if ( '1' == $comment_moderation ) echo ' checked="checked"'; ?> />
    <label for="comment_moderation"><?php _e( 'An administrator must always approve the comment' ); // this is in the default domain ?></label>
    <br/>
    <input type="checkbox" name="comment_whitelist" id="comment_whitelist" value="1"<?php if ( '1' == $comment_whitelist ) echo ' checked="checked"'; ?> />
    <label for="comment_whitelist"><?php _e( 'Comment author must have a previously approved comment' ); // this is in the default domain ?></label>
    <?php
  }

  /** */
  function sanitize__comment_email_verify ( $settings )
  {
    return array ( 'template' => array ( 'subject' => $settings['template']['subject'], 'body' => $settings['template']['body'], ), );
  }

  /** */
  function sanitize_bool ( $settings )
  {
    return ( '1' == $settings ) ? '1' : '';
  }
  
  /**
   * hooked to {@link http://codex.wordpress.org/Plugin_API/Filter_Reference WordPress filter}: {@link http://codex.wordpress.org/Plugin_API/Filter_Reference/plugin_action_links plugin_action_links}
   * @since 0.1.2
   */
  function plugin_action_links ( $links )
  {
    $links[] = sprintf ( '<a href="options-general.php?page=comment-email-verify">%s</a>', __( 'Settings' ) ); // 'Settings' is in the default domain!
    return $links;
  }

  /** hooked to {@link http://codex.wordpress.org/Plugin_API/Action_Reference WordPress action}: {@link http://codex.wordpress.org/Plugin_API/Action_Reference/admin_init admin_init} */
  function admin_init ()
  {
    add_settings_section ( 'comment-email-verify_template', __( 'Message template', 'comment-email-verify' ), array ( 'comment_email_verify', 'settings_section__template' ), 'comment-email-verify' );
    add_settings_field ( 'comment-email-verify_template_subject', __( 'Subject', 'comment-email-verify' ), array ( 'comment_email_verify', 'settings_field__template__subject' ), 'comment-email-verify', 'comment-email-verify_template' );
    add_settings_field ( 'comment-email-verify_template_body', __( 'Message body', 'comment-email-verify' ), array ( 'comment_email_verify', 'settings_field__template__body' ), 'comment-email-verify', 'comment-email-verify_template' );
    
    add_settings_section ( 'comment-email-verify_core-options', __( 'WordPress core settings', 'comment_email_verify' ), array ( 'comment_email_verify', 'settings_section__core_options' ), 'comment-email-verify' );
    add_settings_field ( 'comment-email-verify_core-options_other', __( 'Other comment settings' ), array ( 'comment_email_verify', 'settings_field__core_options__other' ), 'comment-email-verify', 'comment-email-verify_core-options' );
    add_settings_field ( 'comment-email-verify_core-options_email', __( 'E-mail me whenever' ), array ( 'comment_email_verify', 'settings_field__core_options__email' ), 'comment-email-verify', 'comment-email-verify_core-options' );
    add_settings_field ( 'comment-email-verify_core-options_before', __( 'Before a comment appears' ), array ( 'comment_email_verify', 'settings_field__core_options__before' ), 'comment-email-verify', 'comment-email-verify_core-options' );
    
    register_setting ( 'comment-email-verify', 'comment-email-verify', array ( 'comment_email_verify', 'sanitize__comment_email_verify' ) );
    register_setting ( 'comment-email-verify', 'require_name_email', array ( 'comment_email_verify', 'sanitize_bool' ) );
    register_setting ( 'comment-email-verify', 'comment_registration', array ( 'comment_email_verify', 'sanitize_bool' ) );
    register_setting ( 'comment-email-verify', 'comments_notify', array ( 'comment_email_verify', 'sanitize_bool' ) );
    register_setting ( 'comment-email-verify', 'moderation_notify', array ( 'comment_email_verify', 'sanitize_bool' ) );
    register_setting ( 'comment-email-verify', 'comment_moderation', array ( 'comment_email_verify', 'sanitize_bool' ) );
    register_setting ( 'comment-email-verify', 'comment_whitelist', array ( 'comment_email_verify', 'sanitize_bool' ) );

    add_filter ( 'plugin_action_links_' . plugin_basename ( __FILE__ ), array ( 'comment_email_verify', 'plugin_action_links' ) );
  }

  /** */
  function options_page ()
  {
    ?>
    <div class="wrap">
      <div id="icon-options-general" class="icon32"><br></div>
      <h2><?php _e( 'Comment E-Mail Verification Settings', 'comment-email-verify' ); ?></h2>
      <form method="post" action="options.php">
        <?php do_settings_sections ( 'comment-email-verify' ); ?>
        <?php settings_fields ( 'comment-email-verify' ); ?>
        <p class="submit"><input class="button-primary" type="submit" value="<?php esc_attr_e( 'Save Changes' ); // this is in the default domain! ?>" /></p>
      </form>
    </div>
    <?php
  }

  /** hooked to {@link http://codex.wordpress.org/Plugin_API/Action_Reference WordPress action}: {@link http://codex.wordpress.org/Plugin_API/Action_Reference/admin_menu admin_menu} */
  function admin_menu ()
  {
    $page = add_options_page ( __( 'Comment E-Mail Verification Settings', 'comment-email-verify' ), __( 'Comment E-Mail Verification', 'comment-email-verify' ), 'manage_options', 'comment-email-verify', array ( 'comment_email_verify', 'options_page' ) );
    /* translators: %1$s is the plugin name */
    $help = sprintf ( __( '<p><em>%1$s</em> sends an e-mail message to the comment author when a comment is held for moderation. The e-mail message contains a link to verify the comment author\'s e-mail address. When the comment author clicks on that link the comment gets approved immediately. This makes discussions more lively as users don\'t have to wait for the blog admin to approve the comment.</p><p>You can customize the e-mail message using a number of <a href="http://ten-fingers-and-a-brain.com/wordpress-plugins/comment-email-verify/shortcodes/">shortcodes</a>.</p><p>Please note: the e-mail message is sent to the <em>comment</em> author, not the post author.</p><p>A number of factors can influence e-mail delivery to the comment authors. The messages might not be received at all or caught by spam filters. <a href="http://ten-fingers-and-a-brain.com/wordpress-plugins/comment-email-verify/troubleshooting/">Troubleshoot e-mail delivery</a></p>', 'comment-email-verify' ), __( 'Comment E-Mail Verification', 'comment-email-verify' ) );
    add_contextual_help ( $page, $help );
  }

} // class comment_email_verify

add_action ( 'init', array ( 'comment_email_verify', 'init' ) );
add_action ( 'admin_init', array ( 'comment_email_verify', 'admin_init' ) );
add_action ( 'admin_menu', array ( 'comment_email_verify', 'admin_menu' ) );
